;-------------------------------------------------------------------------------
; sys_memory.asm
;
; (c) Texas Instruments 2009-2010, All rights reserved.
;

    .text
    .arm


;-------------------------------------------------------------------------------
; Run Memory Test

    .def     _memoryTest_
    .ref     memoryPort0TestFailNotification
    .ref     memoryPort1TestFailNotification
    .asmfunc

_memoryTest_

        ldr   r12, regPbist    ; Pbist register frame base pointer 
        ldr   r11, palgo       ; Algo array pointer
        ldr   r10, prinfo      ; Rinfo array pointer
        ldr   r9,  regMstGcr   ; MstGcr register pointer 
        mov   r8,  #16         ; Algo loop counter
aloop 
        subs  r8, r8, #1
        beq   pbistEnd
        ldr   r4, [r11], #4    ; load current algo selection
        ldr   r5, [r10], #4    ; load current info selection
        cmp   r4, #0
        beq   aloop
        mov   r7, #1
        str   r7, [r9, #8]    ; enable BIST controller
        movw  r7, #0x10A
        str   r7, [r9]        ; set ROMCLK divider and enable PBIST Test; [9:8]=[10], 
        nop                   ; wait for 32 VBUS cycles
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        mov   r7, #3
        str   r7, [r12, #0x80] ; Pbist PACT register
        str   r4, [r12, #0xC4] ; Select algorithm    
        str   r5, [r12, #0xC8] ; Select test Rams   
        mov   r6, #0
        str   r6, [r12, #0xCC] ; Select test Rams    
        str   r6, [r12, #0x88] ; Disable override    
        str   r7, [r12, #0xC0] ; Use algorithm and rinfo from ROM 
        mov   r7, #1
        str   r7, [r9,  #0x10] ; Clear Pbist done flag
        mov   r6, #0x14
        str   r6, [r12, #0x64] ; Enable ROM based testing and start test  
dloop  
        ldr   r6, [r9, #0x10] ; Load Pbist done flag
        cmp   r6, #0
        beq   dloop        
        ldr   r4, [r12, #0x90] ; read Pbist FSRF0
        ldr   r5, [r12, #0x94] ; read Pbist FSRF1
        adds  r6, r4, r5, lsl #1
        beq   dTest
        ldrb  r0, [r12, #0x63] ; read Pbist RAMT   ;little endianess 
        ldrb  r1, [r12, #0x62] ; read Pbist RAMT   ;little endianess 
;        ldrb  r0, [r12, #0x60] ; read Pbist RAMT  ;big endianess
;        ldrb  r1, [r12, #0x61] ; read Pbist RAMT  ;big endianess 
        tst   r4, #1
port0          
        ldr   r2, [r12, #0xA0] ; read Pbist FSRA0 
        ldr   r3, [r12, #0xA8] ; read Pbist FSRDL0 
        beq   port1
        stmfd sp!, {r0-r12, lr}
        blx   memoryPort0TestFailNotification
        ldmfd sp!, {r0-r12, lr}
port1
        ldr   r2, [r12, #0xA4] ; read Pbist FSRA1 
        ldr   r3, [r12, #0xB0] ; read Pbist FSRDL1 
        tst   r5, #1
        beq   resume
        stmfd sp!, {r0-r12, lr}
        blx   memoryPort1TestFailNotification
        ldmfd sp!, {r0-r12, lr}
resume
        mov   r7, #1
        str   r7, [r9,  #0x10] ; Clear Pbist done flag
        mov   r6, #2
        str   r6, [r12, #0x6C] ; resume Pbist test
        bne   dloop
dTest        
        movw  r4, #0x105
        str   r4, [r9]        ; disable PBIST Test
        b     aloop
pbistEnd
        mov   r7, #0
        str   r7, [r12, #0x80] ; Disable Pbist and ROM slock in PACT register
        bx    lr

regMstGcr     .word 0xFFFFFF58
regPbist      .word 0xFFFFE500

algo
              .word 1<<0
              .word 1<<1
              .word 0<<2
              .word 0<<3
              .word 0<<4
              .word 0<<5
              .word 0<<6
              .word 0<<7
              .word 0<<8
              .word 0<<9
              .word 0<<10
              .word 0<<11
              .word 0<<12
              .word 0<<13
              .word 0<<14
              .word 0<<15
palgo         .word algo

rinfo
	          .word 0x00000003
	          .word 0x00000003
	          .word 0x000020FC
	          .word 0x00001F00
	          .word 0x000020FC
	          .word 0x00001F00
	          .word 0x000020FC
	          .word 0x00001F00
	          .word 0x000020FC
	          .word 0x00001F00
	          .word 0x000020FC
	          .word 0x00001F00
	          .word 0x000020FC
	          .word 0x00001D00
	          .word 0x00000200
	          .word 0x00000200
prinfo        .word rinfo    	

    .endasmfunc


;-------------------------------------------------------------------------------
; Initialize memory

    .def     _memoryInit_
    .asmfunc

_memoryInit_
        
        ldr   r12, regMinitGcr    ; MINITGCR register pointer 
        mov   r4, #0xA            ;Global memory hardware initialization key is enabled
        str   r4, [r12]
        ldr   r4, ramInitMask     ; load RAM initialization mask
        str   r4, [r12, #4]
mloop
        ldr   r5, [r12, #12]      
        tst   r5, #0x100
        beq   mloop
        mov   r4, #5
        str   r4, [r12]
        bx    lr
    
ramInitMask   .word 0x457F
regMinitGcr   .word 0xFFFFFF5C         ;Memory Hardware Initialization Global Control Register (MINITGCR)

    .endasmfunc

    

;-------------------------------------------------------------------------------

