/** @file system.c 
*   @brief System Driver Source File
*   @date 21.July.2009
*   @version 1.01.000
*
*   This file contains:
*   - API Funcions
*   .
*   which are relevant for the System driver.
*/

/* (c) Texas Instruments 2009-2010, All rights reserved. */

/* USER CODE BEGIN (0) */
/* USER CODE END */


/* Include Files */

#include "system.h"

/* USER CODE BEGIN (1) */
/* USER CODE END */


/** @fn void systemInit(void)
*   @brief Initializes System Driver
*
*   This function initializes the System driver.
*
*/

/* USER CODE BEGIN (2) */
/* USER CODE END */

void systemInit(void)
{
/* USER CODE BEGIN (3) */
/* USER CODE END */
    unsigned int i; 
    /** @b Initialize @b Flash @b Wrapper: */

    /** - Setup flash read mode, address wait states and data wait states */
    flashWREG->FRDCNTL =  0x01000000U 
                       | (3U << 8U) 
                       | (1U << 4U) 
                       |  1U;

    /** - Setup flash bank power modes; 0x05050000 */
    flashWREG->FBFALLBACK = (SYS_SLEEP << 14U) 
                          | (SYS_SLEEP << 12U) 
                          | (SYS_SLEEP << 10U) 
                          | (SYS_SLEEP << 8U) 
                          | (SYS_ACTIVE << 6U) 
                          | (SYS_ACTIVE << 4U) 
                          | (SYS_ACTIVE << 2U) 
                          |  SYS_ACTIVE;

    /*check ECLK GIO fucntion*/
    systemREG1->SYSPC1 = 0x00000000U;  /* Configure ECLK as GIO*/
    systemREG1->SYSPC2 = 0x00000001U;  /*GIO OUPUT*/
 while(0)
 {
    systemREG1->SYSPC4 = 0x00000001U;  /*GIO OUPUT 1*/
    for ( i=1; i<30000; i++);
    systemREG1->SYSPC4 = 0x00000000U;  /*GIO OUPUT 0*/
 }      
    
    /** Configure ECLK as cock output. Normally CLK is 80MHz for my application */
    systemREG1->SYSPC1 = 0x00000001U;
//    SYSTEM_1->ECPCNTL |= (0x3 << 23);  /* configure ECLK to use OSCIN */  
    systemREG1->ECPCNTL |= (0x01 << 23);  /* configure ECLK to use VCLK */  
    systemREG1->CLKTEST |= (0x5 << 16);  /* Clock going to ECLK pin is enabled */
    

    /** @b Initialize @b Lpo: */

    systemREG1->LPOMONCTL = (1U << 24U)
                          | (8U << 8U)
                          |  8U;

    /** @b Initialize @b Secondary Pll: */

    systemREG2->PLLCTL3 = ((2U - 1U) << 22U)
                        | ((10U - 1U) << 8U)
                        |  (1U - 1U);


    /** @b Initialize @b Pll: */

    /** - Setup pll control register 1:
    *     - Setup reset on oscillator slip 
    *     - Setup bypass on pll slip
    *     - Setup Pll output clock divider
    *     - Setup reset on oscillator fail
    *     - Setup reference clock divider 
    *     - Setup Pll multiplier          
    */
  /** FMzPLL
    *   PLL_CLK = foscin/NR * NF/OD  
    * 
    *   R=PLLDIV+1 = 0+1=1 
    *   fpll_clk = fpost_clk/R = fpost_clk
    * 
    *   NR=refclkdiv+1 = 5+1 =6
    *   fint_clk = foscin/NR = foscin/6
    *
    *   NF=(PLLMUL/256)+1 = 119 + 1= 120
    *   fvco_clk = fint_clk*NF = foscin/6*120=foscin*20
    *            = 16MHz * 20 = 320MHz
    * 
    *   OD is defined in PLLCTL2, OD=1+1=2
    *   fpost_clk = fvco_clk/2 = 160MHz
    *  
    *   so PLL_CLK = fpost_clk = 160Mhz
    */
 /*      
    systemREG1->PLLCTL1 =  0x00000000U 
                        |  0x20000000U 
                        | (0U << 24U) 
                        |  0x00000000U 
                        | (5U << 16U) 
                        | (119U << 8U);
*/
   systemREG1->PLLCTL1 =  0x20057700;  /*160MHz HCLK*/
 
 	// Clearing the Slip Bit
   systemREG1->GBLSTAT = 0x300;

    /** - Setup pll control register 1 
    *     - Enable/Disable frequency modulation
    *     - Setup spreading rate
    *     - Setup bandwidth adjustment
    *     - Setup internal Pll output divider
    *     - Setup spreading amount
    */
/*
    systemREG1->PLLCTL2 = 0x00000000U
                        | (255U << 22U)
                        | (0U << 12U)
                        | (1U << 9U)
                        |  61U;
*/
    systemREG1->PLLCTL2 = 0x3FC0723D;  /*160MHz, VCLK*/
    
    /** @b Initialize @b Clock @b Tree: */

    /** - Start clock source lock */
    systemREG1->CSDIS = 0x00000000U 
                    | 0x00000000U 
                    | 0x00000000U 
                    | 0x00000008U 
                    | 0x00000004U 
                    | 0x00000000U 
                    | 0x00000000U;

    /** - Wait for until clocks are locked */
    while ((systemREG1->CSVSTAT & ((systemREG1->CSDIS ^ 0xFF) & 0xFF)) != ((systemREG1->CSDIS ^ 0xFF) & 0xFF));

    /** - Setup GCLK, HCLK and VCLK clock source for normal operation, power down mode and after wakeup */
    systemREG1->GHVSRC = (SYS_OSC << 24U) 
                       | (SYS_PLL << 16U) 
                       |  SYS_PLL;

    /** - Power-up all peripharals */
    pcrREG->PSPWRDWNCLR0 = 0xFFFFFFFFU;
    pcrREG->PSPWRDWNCLR1 = 0xFFFFFFFFU;
    pcrREG->PSPWRDWNCLR2 = 0xFFFFFFFFU;
    pcrREG->PSPWRDWNCLR3 = 0xFFFFFFFFU;


    /** - Setup synchronous peripheral clock dividers for VCLK1 and VCLK2 */
    systemREG1->VCLKR  = 1U;
    systemREG1->VCLK2R = 1U;
 //   systemREG1->VCLKR  = 1U;

    /** - Setup RTICLK1 and RTICLK2 clocks */
    systemREG1->RCLKSRC = (1U << 24U)
                        | (SYS_VCLK << 16U) 
                        | (1U << 8U)  
                        |  SYS_VCLK;

    /** - Setup asynchronous peripheral clock sources for AVCLK1 and AVCLK2 */
    systemREG1->VCLKASRC = (SYS_VCLK << 8U)
                         |  SYS_VCLK;

    /** - Enable Peripherals */
    systemREG1->PENA = 1U;

    /** - set ECLK pins functional mode */
    systemREG1->SYSPC1 = 1U;

    /** @note: HCLK >= VCLK2 >= VCLK_sys */
  /* System Module 2*/
   /** - Setup synchronous peripheral clock dividers for VCLK1 and VCLK2 */
 
    /** - Setup asynchronous peripheral clock sources and dividers for AVCLK3 and AVCLK4 */
    systemREG2->VCLKACON1 =   (SYS_VCLK << 16U) 
                          | (SYS_VCLK << 0U)  
                          | ( 3 << 24)
                          | ( 3 << 8);          

/* USER CODE BEGIN (4) */
    systemREG1->GPREG1 |= (0x1 << 31);

	/* Set PinMuxing */
	/* Unlock pinmxing registers */
//	Gladiator_IOMM_UNLOCK();
	
	/* Set muxing for modules used */
//    Gladiator_PINMUX_USB();
	// Gladiator_PINMUX_AD2_ALT_TRIGGER_SOURCE();
	
	/* Lock pinmux registers */
//	Gladiator_IOMM_LOCK();
	
/* USER CODE END */
}

void systemPowerDown(unsigned mode)
{
    /* Disable clock sources */
    systemREG1->CSDISSET = mode & 0x000000FFU;

    /* Disable clock domains */
    systemREG1->CSDDIS = (mode >> 8U) & 0x00000FFFU;

    /* Idle CPU */
    asm(" wfi");
}


/* USER CODE BEGIN (5) */
/* USER CODE END */
